/*******************************************************************************
* File Name: SDCARD.c
* Version 2.50
*
* Description:
*  This file provides all API functionality of the SPI Master component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SDCARD_PVT.h"

#if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
    volatile uint8 SDCARD_txBuffer[SDCARD_TX_BUFFER_SIZE];
    volatile uint8 SDCARD_txBufferFull;
    volatile uint8 SDCARD_txBufferRead;
    volatile uint8 SDCARD_txBufferWrite;
#endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

#if(SDCARD_RX_SOFTWARE_BUF_ENABLED)
    volatile uint8 SDCARD_rxBuffer[SDCARD_RX_BUFFER_SIZE];
    volatile uint8 SDCARD_rxBufferFull;
    volatile uint8 SDCARD_rxBufferRead;
    volatile uint8 SDCARD_rxBufferWrite;
#endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

uint8 SDCARD_initVar = 0u;

volatile uint8 SDCARD_swStatusTx;
volatile uint8 SDCARD_swStatusRx;


/*******************************************************************************
* Function Name: SDCARD_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default SPIM configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  When this function is called it initializes all of the necessary parameters
*  for execution. i.e. setting the initial interrupt mask, configuring the
*  interrupt service routine, configuring the bit-counter parameters and
*  clearing the FIFO and Status Register.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_Init(void) 
{
    /* Initialize the Bit counter */
    SDCARD_COUNTER_PERIOD_REG = SDCARD_BITCTR_INIT;

    /* Init TX ISR  */
    #if(0u != SDCARD_INTERNAL_TX_INT_ENABLED)
        CyIntDisable         (SDCARD_TX_ISR_NUMBER);
        CyIntSetPriority     (SDCARD_TX_ISR_NUMBER,  SDCARD_TX_ISR_PRIORITY);
        (void) CyIntSetVector(SDCARD_TX_ISR_NUMBER, &SDCARD_TX_ISR);
    #endif /* (0u != SDCARD_INTERNAL_TX_INT_ENABLED) */

    /* Init RX ISR  */
    #if(0u != SDCARD_INTERNAL_RX_INT_ENABLED)
        CyIntDisable         (SDCARD_RX_ISR_NUMBER);
        CyIntSetPriority     (SDCARD_RX_ISR_NUMBER,  SDCARD_RX_ISR_PRIORITY);
        (void) CyIntSetVector(SDCARD_RX_ISR_NUMBER, &SDCARD_RX_ISR);
    #endif /* (0u != SDCARD_INTERNAL_RX_INT_ENABLED) */

    /* Clear any stray data from the RX and TX FIFO */
    SDCARD_ClearFIFO();

    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)
        SDCARD_rxBufferFull  = 0u;
        SDCARD_rxBufferRead  = 0u;
        SDCARD_rxBufferWrite = 0u;
    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        SDCARD_txBufferFull  = 0u;
        SDCARD_txBufferRead  = 0u;
        SDCARD_txBufferWrite = 0u;
    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    (void) SDCARD_ReadTxStatus(); /* Clear Tx status and swStatusTx */
    (void) SDCARD_ReadRxStatus(); /* Clear Rx status and swStatusRx */

    /* Configure TX and RX interrupt mask */
    SDCARD_TX_STATUS_MASK_REG = SDCARD_TX_INIT_INTERRUPTS_MASK;
    SDCARD_RX_STATUS_MASK_REG = SDCARD_RX_INIT_INTERRUPTS_MASK;
}


/*******************************************************************************
* Function Name: SDCARD_Enable
********************************************************************************
*
* Summary:
*  Enable SPIM component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void SDCARD_Enable(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    SDCARD_COUNTER_CONTROL_REG |= SDCARD_CNTR_ENABLE;
    SDCARD_TX_STATUS_ACTL_REG  |= SDCARD_INT_ENABLE;
    SDCARD_RX_STATUS_ACTL_REG  |= SDCARD_INT_ENABLE;
    CyExitCriticalSection(enableInterrupts);

    #if(0u != SDCARD_INTERNAL_CLOCK)
        SDCARD_IntClock_Enable();
    #endif /* (0u != SDCARD_INTERNAL_CLOCK) */

    SDCARD_EnableTxInt();
    SDCARD_EnableRxInt();
}


/*******************************************************************************
* Function Name: SDCARD_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_initVar - used to check initial configuration, modified on
*  first function call.
*
* Theory:
*  Enable the clock input to enable operation.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_Start(void) 
{
    if(0u == SDCARD_initVar)
    {
        SDCARD_Init();
        SDCARD_initVar = 1u;
    }

    SDCARD_Enable();
}


/*******************************************************************************
* Function Name: SDCARD_Stop
********************************************************************************
*
* Summary:
*  Disable the SPI Master component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the clock input to enable operation.
*
*******************************************************************************/
void SDCARD_Stop(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    SDCARD_TX_STATUS_ACTL_REG &= ((uint8) ~SDCARD_INT_ENABLE);
    SDCARD_RX_STATUS_ACTL_REG &= ((uint8) ~SDCARD_INT_ENABLE);
    CyExitCriticalSection(enableInterrupts);

    #if(0u != SDCARD_INTERNAL_CLOCK)
        SDCARD_IntClock_Disable();
    #endif /* (0u != SDCARD_INTERNAL_CLOCK) */

    SDCARD_DisableTxInt();
    SDCARD_DisableRxInt();
}


/*******************************************************************************
* Function Name: SDCARD_EnableTxInt
********************************************************************************
*
* Summary:
*  Enable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_EnableTxInt(void) 
{
    #if(0u != SDCARD_INTERNAL_TX_INT_ENABLED)
        CyIntEnable(SDCARD_TX_ISR_NUMBER);
    #endif /* (0u != SDCARD_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_EnableRxInt
********************************************************************************
*
* Summary:
*  Enable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_EnableRxInt(void) 
{
    #if(0u != SDCARD_INTERNAL_RX_INT_ENABLED)
        CyIntEnable(SDCARD_RX_ISR_NUMBER);
    #endif /* (0u != SDCARD_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_DisableTxInt
********************************************************************************
*
* Summary:
*  Disable internal Tx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Tx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_DisableTxInt(void) 
{
    #if(0u != SDCARD_INTERNAL_TX_INT_ENABLED)
        CyIntDisable(SDCARD_TX_ISR_NUMBER);
    #endif /* (0u != SDCARD_INTERNAL_TX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_DisableRxInt
********************************************************************************
*
* Summary:
*  Disable internal Rx interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal Rx interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_DisableRxInt(void) 
{
    #if(0u != SDCARD_INTERNAL_RX_INT_ENABLED)
        CyIntDisable(SDCARD_RX_ISR_NUMBER);
    #endif /* (0u != SDCARD_INTERNAL_RX_INT_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_SetTxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SDCARD_SetTxInterruptMode(uint8 intSrc) 
{
    SDCARD_TX_STATUS_MASK_REG = intSrc;
}


/*******************************************************************************
* Function Name: SDCARD_SetRxInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SDCARD_SetRxInterruptMode(uint8 intSrc) 
{
    SDCARD_RX_STATUS_MASK_REG  = intSrc;
}


/*******************************************************************************
* Function Name: SDCARD_ReadTxStatus
********************************************************************************
*
* Summary:
*  Read the Tx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Tx status register.
*
* Global variables:
*  SDCARD_swStatusTx - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Tx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SDCARD_ReadTxStatus(void) 
{
    uint8 tmpStatus;

    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SDCARD_DisableTxInt();

        tmpStatus = SDCARD_GET_STATUS_TX(SDCARD_swStatusTx);
        SDCARD_swStatusTx = 0u;

        SDCARD_EnableTxInt();

    #else

        tmpStatus = SDCARD_TX_STATUS_REG;

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: SDCARD_ReadRxStatus
********************************************************************************
*
* Summary:
*  Read the Rx status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the Rx status register.
*
* Global variables:
*  SDCARD_swStatusRx - used to store in software Rx status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the Rx status register for error
*  detection and flow control.
*
* Side Effects:
*  Clear Rx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SDCARD_ReadRxStatus(void) 
{
    uint8 tmpStatus;

    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        SDCARD_DisableRxInt();

        tmpStatus = SDCARD_GET_STATUS_RX(SDCARD_swStatusRx);
        SDCARD_swStatusRx = 0u;

        SDCARD_EnableRxInt();

    #else

        tmpStatus = SDCARD_RX_STATUS_REG;

    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/*******************************************************************************
* Function Name: SDCARD_WriteTxData
********************************************************************************
*
* Summary:
*  Write a byte of data to be sent across the SPI.
*
* Parameters:
*  txDataByte: The data value to send across the SPI.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call if TX Software Buffer is used.
*  SDCARD_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*  SDCARD_txBuffer[SDCARD_TX_BUFFER_SIZE] - used to store
*  data to sending, modified every function call if TX Software Buffer is used.
*
* Theory:
*  Allows the user to transmit any byte of data in a single transfer.
*
* Side Effects:
*  If this function is called again before the previous byte is finished then
*  the next byte will be appended to the transfer with no time between
*  the byte transfers. Clear Tx status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_WriteTxData(uint8 txData) 
{
    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)

        uint8 tempStatus;
        uint8 tmpTxBufferRead;

        /* Block if TX buffer is FULL: don't overwrite */
        do
        {
            tmpTxBufferRead = SDCARD_txBufferRead;
            if(0u == tmpTxBufferRead)
            {
                tmpTxBufferRead = (SDCARD_TX_BUFFER_SIZE - 1u);
            }
            else
            {
                tmpTxBufferRead--;
            }

        }while(tmpTxBufferRead == SDCARD_txBufferWrite);

        /* Disable TX interrupt to protect global veriables */
        SDCARD_DisableTxInt();

        tempStatus = SDCARD_GET_STATUS_TX(SDCARD_swStatusTx);
        SDCARD_swStatusTx = tempStatus;


        if((SDCARD_txBufferRead == SDCARD_txBufferWrite) &&
           (0u != (SDCARD_swStatusTx & SDCARD_STS_TX_FIFO_NOT_FULL)))
        {
            /* Put data element into the TX FIFO */
            CY_SET_REG8(SDCARD_TXDATA_PTR, txData);
        }
        else
        {
            /* Add to the TX software buffer */
            SDCARD_txBufferWrite++;
            if(SDCARD_txBufferWrite >= SDCARD_TX_BUFFER_SIZE)
            {
                SDCARD_txBufferWrite = 0u;
            }

            if(SDCARD_txBufferWrite == SDCARD_txBufferRead)
            {
                SDCARD_txBufferRead++;
                if(SDCARD_txBufferRead >= SDCARD_TX_BUFFER_SIZE)
                {
                    SDCARD_txBufferRead = 0u;
                }
                SDCARD_txBufferFull = 1u;
            }

            SDCARD_txBuffer[SDCARD_txBufferWrite] = txData;

            SDCARD_TX_STATUS_MASK_REG |= SDCARD_STS_TX_FIFO_NOT_FULL;
        }

        SDCARD_EnableTxInt();

    #else
        /* Wait until TX FIFO has a place */
        while(0u == (SDCARD_TX_STATUS_REG & SDCARD_STS_TX_FIFO_NOT_FULL))
        {
        }

        /* Put data element into the TX FIFO */
        CY_SET_REG8(SDCARD_TXDATA_PTR, txData);

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_ReadRxData
********************************************************************************
*
* Summary:
*  Read the next byte of data received across the SPI.
*
* Parameters:
*  None.
*
* Return:
*  The next byte of data read from the FIFO.
*
* Global variables:
*  SDCARD_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  SDCARD_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function
*  call if RX Software Buffer is used.
*  SDCARD_rxBuffer[SDCARD_RX_BUFFER_SIZE] - used to store
*  received data.
*
* Theory:
*  Allows the user to read a byte of data received.
*
* Side Effects:
*  Will return invalid data if the FIFO is empty. The user should Call
*  GetRxBufferSize() and if it returns a non-zero value then it is safe to call
*  ReadByte() function.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SDCARD_ReadRxData(void) 
{
    uint8 rxData;

    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        SDCARD_DisableRxInt();

        if(SDCARD_rxBufferRead != SDCARD_rxBufferWrite)
        {
            if(0u == SDCARD_rxBufferFull)
            {
                SDCARD_rxBufferRead++;
                if(SDCARD_rxBufferRead >= SDCARD_RX_BUFFER_SIZE)
                {
                    SDCARD_rxBufferRead = 0u;
                }
            }
            else
            {
                SDCARD_rxBufferFull = 0u;
            }
        }

        rxData = SDCARD_rxBuffer[SDCARD_rxBufferRead];

        SDCARD_EnableRxInt();

    #else

        rxData = CY_GET_REG8(SDCARD_RXDATA_PTR);

    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    return(rxData);
}


/*******************************************************************************
* Function Name: SDCARD_GetRxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the RX buffer.
*  If RX Software Buffer not used then function return 0 if FIFO empty or 1 if
*  FIFO not empty. In another case function return size of RX Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the RX buffer.
*
* Global variables:
*  SDCARD_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer.
*  SDCARD_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8 SDCARD_GetRxBufferSize(void) 
{
    uint8 size;

    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)

        /* Disable RX interrupt to protect global veriables */
        SDCARD_DisableRxInt();

        if(SDCARD_rxBufferRead == SDCARD_rxBufferWrite)
        {
            size = 0u;
        }
        else if(SDCARD_rxBufferRead < SDCARD_rxBufferWrite)
        {
            size = (SDCARD_rxBufferWrite - SDCARD_rxBufferRead);
        }
        else
        {
            size = (SDCARD_RX_BUFFER_SIZE - SDCARD_rxBufferRead) + SDCARD_rxBufferWrite;
        }

        SDCARD_EnableRxInt();

    #else

        /* We can only know if there is data in the RX FIFO */
        size = (0u != (SDCARD_RX_STATUS_REG & SDCARD_STS_RX_FIFO_NOT_EMPTY)) ? 1u : 0u;

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: SDCARD_GetTxBufferSize
********************************************************************************
*
* Summary:
*  Returns the number of bytes/words of data currently held in the TX buffer.
*  If TX Software Buffer not used then function return 0 - if FIFO empty, 1 - if
*  FIFO not full, 4 - if FIFO full. In another case function return size of TX
*  Software Buffer.
*
* Parameters:
*  None.
*
* Return:
*  Integer count of the number of bytes/words in the TX buffer.
*
* Global variables:
*  SDCARD_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  SDCARD_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
uint8  SDCARD_GetTxBufferSize(void) 
{
    uint8 size;

    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SDCARD_DisableTxInt();

        if(SDCARD_txBufferRead == SDCARD_txBufferWrite)
        {
            size = 0u;
        }
        else if(SDCARD_txBufferRead < SDCARD_txBufferWrite)
        {
            size = (SDCARD_txBufferWrite - SDCARD_txBufferRead);
        }
        else
        {
            size = (SDCARD_TX_BUFFER_SIZE - SDCARD_txBufferRead) + SDCARD_txBufferWrite;
        }

        SDCARD_EnableTxInt();

    #else

        size = SDCARD_TX_STATUS_REG;

        if(0u != (size & SDCARD_STS_TX_FIFO_EMPTY))
        {
            size = 0u;
        }
        else if(0u != (size & SDCARD_STS_TX_FIFO_NOT_FULL))
        {
            size = 1u;
        }
        else
        {
            size = SDCARD_FIFO_SIZE;
        }

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    return(size);
}


/*******************************************************************************
* Function Name: SDCARD_ClearRxBuffer
********************************************************************************
*
* Summary:
*  Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer, modified every function
*  call - resets to zero.
*  SDCARD_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any received data not read from the RAM buffer will be lost when overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_ClearRxBuffer(void) 
{
    /* Clear Hardware RX FIFO */
    while(0u !=(SDCARD_RX_STATUS_REG & SDCARD_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG8(SDCARD_RXDATA_PTR);
    }

    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)
        /* Disable RX interrupt to protect global veriables */
        SDCARD_DisableRxInt();

        SDCARD_rxBufferFull  = 0u;
        SDCARD_rxBufferRead  = 0u;
        SDCARD_rxBufferWrite = 0u;

        SDCARD_EnableRxInt();
    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */
}


/*******************************************************************************
* Function Name: SDCARD_ClearTxBuffer
********************************************************************************
*
* Summary:
*  Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer, modified every function
*  call - resets to zero.
*  SDCARD_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified every function call -
*  resets to zero.
*
* Theory:
*  Setting the pointers to zero makes the system believe there is no data to
*  read and writing will resume at address 0 overwriting any data that may have
*  remained in the RAM.
*
* Side Effects:
*  Any data not yet transmitted from the RAM buffer will be lost when
*  overwritten.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_ClearTxBuffer(void) 
{
    uint8 enableInterrupts;

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    SDCARD_AUX_CONTROL_DP0_REG |= ((uint8)  SDCARD_TX_FIFO_CLR);
    SDCARD_AUX_CONTROL_DP0_REG &= ((uint8) ~SDCARD_TX_FIFO_CLR);

    #if(SDCARD_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        SDCARD_AUX_CONTROL_DP1_REG |= ((uint8)  SDCARD_TX_FIFO_CLR);
        SDCARD_AUX_CONTROL_DP1_REG &= ((uint8) ~SDCARD_TX_FIFO_CLR);
    #endif /* (SDCARD_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);

    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        /* Disable TX interrupt to protect global veriables */
        SDCARD_DisableTxInt();

        SDCARD_txBufferFull  = 0u;
        SDCARD_txBufferRead  = 0u;
        SDCARD_txBufferWrite = 0u;

        /* Buffer is EMPTY: disable TX FIFO NOT FULL interrupt */
        SDCARD_TX_STATUS_MASK_REG &= ((uint8) ~SDCARD_STS_TX_FIFO_NOT_FULL);

        SDCARD_EnableTxInt();
    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */
}


#if(0u != SDCARD_BIDIRECTIONAL_MODE)
    /*******************************************************************************
    * Function Name: SDCARD_TxEnable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to transmit.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void SDCARD_TxEnable(void) 
    {
        SDCARD_CONTROL_REG |= SDCARD_CTRL_TX_SIGNAL_EN;
    }


    /*******************************************************************************
    * Function Name: SDCARD_TxDisable
    ********************************************************************************
    *
    * Summary:
    *  If the SPI master is configured to use a single bi-directional pin then this
    *  will set the bi-directional pin to receive.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void SDCARD_TxDisable(void) 
    {
        SDCARD_CONTROL_REG &= ((uint8) ~SDCARD_CTRL_TX_SIGNAL_EN);
    }

#endif /* (0u != SDCARD_BIDIRECTIONAL_MODE) */


/*******************************************************************************
* Function Name: SDCARD_PutArray
********************************************************************************
*
* Summary:
*  Write available data from ROM/RAM to the TX buffer while space is available
*  in the TX buffer. Keep trying until all data is passed to the TX buffer.
*
* Parameters:
*  *buffer: Pointer to the location in RAM containing the data to send
*  byteCount: The number of bytes to move to the transmit buffer.
*
* Return:
*  None.
*
* Side Effects:
*  Will stay in this routine until all data has been sent.  May get locked in
*  this loop if data is not being initiated by the master if there is not
*  enough room in the TX FIFO.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void SDCARD_PutArray(const uint8 buffer[], uint8 byteCount)
                                                                          
{
    uint8 bufIndex;

    bufIndex = 0u;

    while(byteCount > 0u)
    {
        SDCARD_WriteTxData(buffer[bufIndex]);
        bufIndex++;
        byteCount--;
    }
}


/*******************************************************************************
* Function Name: SDCARD_ClearFIFO
********************************************************************************
*
* Summary:
*  Clear the RX and TX FIFO's of all data for a fresh start.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
*  Clear status register of the component.
*
*******************************************************************************/
void SDCARD_ClearFIFO(void) 
{
    uint8 enableInterrupts;

    /* Clear Hardware RX FIFO */
    while(0u !=(SDCARD_RX_STATUS_REG & SDCARD_STS_RX_FIFO_NOT_EMPTY))
    {
        (void) CY_GET_REG8(SDCARD_RXDATA_PTR);
    }

    enableInterrupts = CyEnterCriticalSection();
    /* Clear TX FIFO */
    SDCARD_AUX_CONTROL_DP0_REG |= ((uint8)  SDCARD_TX_FIFO_CLR);
    SDCARD_AUX_CONTROL_DP0_REG &= ((uint8) ~SDCARD_TX_FIFO_CLR);

    #if(SDCARD_USE_SECOND_DATAPATH)
        /* Clear TX FIFO for 2nd Datapath */
        SDCARD_AUX_CONTROL_DP1_REG |= ((uint8)  SDCARD_TX_FIFO_CLR);
        SDCARD_AUX_CONTROL_DP1_REG &= ((uint8) ~SDCARD_TX_FIFO_CLR);
    #endif /* (SDCARD_USE_SECOND_DATAPATH) */
    CyExitCriticalSection(enableInterrupts);
}


/* Following functions are for version Compatibility, they are obsolete.
*  Please do not use it in new projects.
*/


/*******************************************************************************
* Function Name: SDCARD_EnableInt
********************************************************************************
*
* Summary:
*  Enable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Enable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_EnableInt(void) 
{
    SDCARD_EnableRxInt();
    SDCARD_EnableTxInt();
}


/*******************************************************************************
* Function Name: SDCARD_DisableInt
********************************************************************************
*
* Summary:
*  Disable internal interrupt generation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Theory:
*  Disable the internal interrupt output -or- the interrupt component itself.
*
*******************************************************************************/
void SDCARD_DisableInt(void) 
{
    SDCARD_DisableTxInt();
    SDCARD_DisableRxInt();
}


/*******************************************************************************
* Function Name: SDCARD_SetInterruptMode
********************************************************************************
*
* Summary:
*  Configure which status bits trigger an interrupt event.
*
* Parameters:
*  intSrc: An or'd combination of the desired status bit masks (defined in the
*  header file).
*
* Return:
*  None.
*
* Theory:
*  Enables the output of specific status bits to the interrupt controller.
*
*******************************************************************************/
void SDCARD_SetInterruptMode(uint8 intSrc) 
{
    SDCARD_TX_STATUS_MASK_REG  = (intSrc & ((uint8) ~SDCARD_STS_SPI_IDLE));
    SDCARD_RX_STATUS_MASK_REG  =  intSrc;
}


/*******************************************************************************
* Function Name: SDCARD_ReadStatus
********************************************************************************
*
* Summary:
*  Read the status register for the component.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the status register.
*
* Global variables:
*  SDCARD_swStatus - used to store in software status register,
*  modified every function call - resets to zero.
*
* Theory:
*  Allows the user and the API to read the status register for error detection
*  and flow control.
*
* Side Effects:
*  Clear status register of the component.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 SDCARD_ReadStatus(void) 
{
    uint8 tmpStatus;

    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED || SDCARD_RX_SOFTWARE_BUF_ENABLED)

        SDCARD_DisableInt();

        tmpStatus  = SDCARD_GET_STATUS_RX(SDCARD_swStatusRx);
        tmpStatus |= SDCARD_GET_STATUS_TX(SDCARD_swStatusTx);
        tmpStatus &= ((uint8) ~SDCARD_STS_SPI_IDLE);

        SDCARD_swStatusTx = 0u;
        SDCARD_swStatusRx = 0u;

        SDCARD_EnableInt();

    #else

        tmpStatus  = SDCARD_RX_STATUS_REG;
        tmpStatus |= SDCARD_TX_STATUS_REG;
        tmpStatus &= ((uint8) ~SDCARD_STS_SPI_IDLE);

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED || SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    return(tmpStatus);
}


/* [] END OF FILE */
