/*******************************************************************************
* File Name: SDCARD.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_SDCARD_H)
#define CY_SPIM_SDCARD_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define SDCARD_INTERNAL_CLOCK             (1u)

#if(0u != SDCARD_INTERNAL_CLOCK)
    #include "SDCARD_IntClock.h"
#endif /* (0u != SDCARD_INTERNAL_CLOCK) */

#define SDCARD_MODE                       (1u)
#define SDCARD_DATA_WIDTH                 (8u)
#define SDCARD_MODE_USE_ZERO              (1u)
#define SDCARD_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define SDCARD_TX_BUFFER_SIZE             (4u)
#define SDCARD_RX_BUFFER_SIZE             (4u)
#define SDCARD_INTERNAL_TX_INT_ENABLED    (0u)
#define SDCARD_INTERNAL_RX_INT_ENABLED    (0u)

#define SDCARD_SINGLE_REG_SIZE            (8u)
#define SDCARD_USE_SECOND_DATAPATH        (SDCARD_DATA_WIDTH > SDCARD_SINGLE_REG_SIZE)

#define SDCARD_FIFO_SIZE                  (4u)
#define SDCARD_TX_SOFTWARE_BUF_ENABLED    ((0u != SDCARD_INTERNAL_TX_INT_ENABLED) && \
                                                     (SDCARD_TX_BUFFER_SIZE > SDCARD_FIFO_SIZE))

#define SDCARD_RX_SOFTWARE_BUF_ENABLED    ((0u != SDCARD_INTERNAL_RX_INT_ENABLED) && \
                                                     (SDCARD_RX_BUFFER_SIZE > SDCARD_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} SDCARD_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  SDCARD_Init(void)                           ;
void  SDCARD_Enable(void)                         ;
void  SDCARD_Start(void)                          ;
void  SDCARD_Stop(void)                           ;

void  SDCARD_EnableTxInt(void)                    ;
void  SDCARD_EnableRxInt(void)                    ;
void  SDCARD_DisableTxInt(void)                   ;
void  SDCARD_DisableRxInt(void)                   ;

void  SDCARD_Sleep(void)                          ;
void  SDCARD_Wakeup(void)                         ;
void  SDCARD_SaveConfig(void)                     ;
void  SDCARD_RestoreConfig(void)                  ;

void  SDCARD_SetTxInterruptMode(uint8 intSrc)     ;
void  SDCARD_SetRxInterruptMode(uint8 intSrc)     ;
uint8 SDCARD_ReadTxStatus(void)                   ;
uint8 SDCARD_ReadRxStatus(void)                   ;
void  SDCARD_WriteTxData(uint8 txData)  \
                                                            ;
uint8 SDCARD_ReadRxData(void) \
                                                            ;
uint8 SDCARD_GetRxBufferSize(void)                ;
uint8 SDCARD_GetTxBufferSize(void)                ;
void  SDCARD_ClearRxBuffer(void)                  ;
void  SDCARD_ClearTxBuffer(void)                  ;
void  SDCARD_ClearFIFO(void)                              ;
void  SDCARD_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != SDCARD_BIDIRECTIONAL_MODE)
    void  SDCARD_TxEnable(void)                   ;
    void  SDCARD_TxDisable(void)                  ;
#endif /* (0u != SDCARD_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(SDCARD_TX_ISR);
CY_ISR_PROTO(SDCARD_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 SDCARD_initVar;


/***************************************
*           API Constants
***************************************/

#define SDCARD_TX_ISR_NUMBER     ((uint8) (SDCARD_TxInternalInterrupt__INTC_NUMBER))
#define SDCARD_RX_ISR_NUMBER     ((uint8) (SDCARD_RxInternalInterrupt__INTC_NUMBER))

#define SDCARD_TX_ISR_PRIORITY   ((uint8) (SDCARD_TxInternalInterrupt__INTC_PRIOR_NUM))
#define SDCARD_RX_ISR_PRIORITY   ((uint8) (SDCARD_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define SDCARD_INT_ON_SPI_DONE    ((uint8) (0u   << SDCARD_STS_SPI_DONE_SHIFT))
#define SDCARD_INT_ON_TX_EMPTY    ((uint8) (0u   << SDCARD_STS_TX_FIFO_EMPTY_SHIFT))
#define SDCARD_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           SDCARD_STS_TX_FIFO_NOT_FULL_SHIFT))
#define SDCARD_INT_ON_BYTE_COMP   ((uint8) (0u  << SDCARD_STS_BYTE_COMPLETE_SHIFT))
#define SDCARD_INT_ON_SPI_IDLE    ((uint8) (0u   << SDCARD_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define SDCARD_INT_ON_TX_NOT_FULL_DEF ((SDCARD_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (SDCARD_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define SDCARD_TX_INIT_INTERRUPTS_MASK    (SDCARD_INT_ON_SPI_DONE  | \
                                                     SDCARD_INT_ON_TX_EMPTY  | \
                                                     SDCARD_INT_ON_TX_NOT_FULL_DEF | \
                                                     SDCARD_INT_ON_BYTE_COMP | \
                                                     SDCARD_INT_ON_SPI_IDLE)

#define SDCARD_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          SDCARD_STS_RX_FIFO_FULL_SHIFT))
#define SDCARD_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          SDCARD_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define SDCARD_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          SDCARD_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define SDCARD_RX_INIT_INTERRUPTS_MASK    (SDCARD_INT_ON_RX_FULL      | \
                                                     SDCARD_INT_ON_RX_NOT_EMPTY | \
                                                     SDCARD_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define SDCARD_BITCTR_INIT            (((uint8) (SDCARD_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define SDCARD_TXDATA_REG (* (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F0_REG)
    #define SDCARD_TXDATA_PTR (  (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F0_REG)
    #define SDCARD_RXDATA_REG (* (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F1_REG)
    #define SDCARD_RXDATA_PTR (  (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(SDCARD_USE_SECOND_DATAPATH)
        #define SDCARD_TXDATA_REG (* (reg16 *) \
                                          SDCARD_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define SDCARD_TXDATA_PTR (  (reg16 *) \
                                          SDCARD_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define SDCARD_RXDATA_REG (* (reg16 *) \
                                          SDCARD_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define SDCARD_RXDATA_PTR (  (reg16 *) \
                                          SDCARD_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define SDCARD_TXDATA_REG (* (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F0_REG)
        #define SDCARD_TXDATA_PTR (  (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F0_REG)
        #define SDCARD_RXDATA_REG (* (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F1_REG)
        #define SDCARD_RXDATA_PTR (  (reg8 *) \
                                                SDCARD_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (SDCARD_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define SDCARD_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        SDCARD_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define SDCARD_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        SDCARD_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(SDCARD_USE_SECOND_DATAPATH)
    #define SDCARD_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        SDCARD_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define SDCARD_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        SDCARD_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (SDCARD_USE_SECOND_DATAPATH) */

#define SDCARD_COUNTER_PERIOD_REG     (* (reg8 *) SDCARD_BSPIM_BitCounter__PERIOD_REG)
#define SDCARD_COUNTER_PERIOD_PTR     (  (reg8 *) SDCARD_BSPIM_BitCounter__PERIOD_REG)
#define SDCARD_COUNTER_CONTROL_REG    (* (reg8 *) SDCARD_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define SDCARD_COUNTER_CONTROL_PTR    (  (reg8 *) SDCARD_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define SDCARD_TX_STATUS_REG          (* (reg8 *) SDCARD_BSPIM_TxStsReg__STATUS_REG)
#define SDCARD_TX_STATUS_PTR          (  (reg8 *) SDCARD_BSPIM_TxStsReg__STATUS_REG)
#define SDCARD_RX_STATUS_REG          (* (reg8 *) SDCARD_BSPIM_RxStsReg__STATUS_REG)
#define SDCARD_RX_STATUS_PTR          (  (reg8 *) SDCARD_BSPIM_RxStsReg__STATUS_REG)

#define SDCARD_CONTROL_REG            (* (reg8 *) \
                                      SDCARD_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define SDCARD_CONTROL_PTR            (  (reg8 *) \
                                      SDCARD_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define SDCARD_TX_STATUS_MASK_REG     (* (reg8 *) SDCARD_BSPIM_TxStsReg__MASK_REG)
#define SDCARD_TX_STATUS_MASK_PTR     (  (reg8 *) SDCARD_BSPIM_TxStsReg__MASK_REG)
#define SDCARD_RX_STATUS_MASK_REG     (* (reg8 *) SDCARD_BSPIM_RxStsReg__MASK_REG)
#define SDCARD_RX_STATUS_MASK_PTR     (  (reg8 *) SDCARD_BSPIM_RxStsReg__MASK_REG)

#define SDCARD_TX_STATUS_ACTL_REG     (* (reg8 *) SDCARD_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define SDCARD_TX_STATUS_ACTL_PTR     (  (reg8 *) SDCARD_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define SDCARD_RX_STATUS_ACTL_REG     (* (reg8 *) SDCARD_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define SDCARD_RX_STATUS_ACTL_PTR     (  (reg8 *) SDCARD_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(SDCARD_USE_SECOND_DATAPATH)
    #define SDCARD_AUX_CONTROLDP1     (SDCARD_AUX_CONTROL_DP1_REG)
#endif /* (SDCARD_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define SDCARD_STS_SPI_DONE_SHIFT             (0x00u)
#define SDCARD_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define SDCARD_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define SDCARD_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define SDCARD_STS_SPI_IDLE_SHIFT             (0x04u)
#define SDCARD_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define SDCARD_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define SDCARD_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define SDCARD_STS_SPI_DONE           ((uint8) (0x01u << SDCARD_STS_SPI_DONE_SHIFT))
#define SDCARD_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << SDCARD_STS_TX_FIFO_EMPTY_SHIFT))
#define SDCARD_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << SDCARD_STS_TX_FIFO_NOT_FULL_SHIFT))
#define SDCARD_STS_BYTE_COMPLETE      ((uint8) (0x01u << SDCARD_STS_BYTE_COMPLETE_SHIFT))
#define SDCARD_STS_SPI_IDLE           ((uint8) (0x01u << SDCARD_STS_SPI_IDLE_SHIFT))
#define SDCARD_STS_RX_FIFO_FULL       ((uint8) (0x01u << SDCARD_STS_RX_FIFO_FULL_SHIFT))
#define SDCARD_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << SDCARD_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define SDCARD_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << SDCARD_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define SDCARD_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define SDCARD_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define SDCARD_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define SDCARD_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define SDCARD_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define SDCARD_FIFO_CLR       (SDCARD_TX_FIFO_CLR | SDCARD_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define SDCARD_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define SDCARD_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define SDCARD_AUX_CTRL_FIFO0_CLR         (0x01u)
#define SDCARD_AUX_CTRL_FIFO1_CLR         (0x02u)
#define SDCARD_AUX_CTRL_FIFO0_LVL         (0x04u)
#define SDCARD_AUX_CTRL_FIFO1_LVL         (0x08u)
#define SDCARD_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define SDCARD_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define SDCARD_IS_ENABLED (0u != (SDCARD_TX_STATUS_ACTL_REG & SDCARD_INT_ENABLE))

/* Retuns TX status register */
#define SDCARD_GET_STATUS_TX(swTxSts) ( (uint8)(SDCARD_TX_STATUS_REG | \
                                                          ((swTxSts) & SDCARD_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define SDCARD_GET_STATUS_RX(swRxSts) ( (uint8)(SDCARD_RX_STATUS_REG | \
                                                          ((swRxSts) & SDCARD_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define SDCARD_WriteByte   SDCARD_WriteTxData
#define SDCARD_ReadByte    SDCARD_ReadRxData
void  SDCARD_SetInterruptMode(uint8 intSrc)       ;
uint8 SDCARD_ReadStatus(void)                     ;
void  SDCARD_EnableInt(void)                      ;
void  SDCARD_DisableInt(void)                     ;

#define SDCARD_TXDATA                 (SDCARD_TXDATA_REG)
#define SDCARD_RXDATA                 (SDCARD_RXDATA_REG)
#define SDCARD_AUX_CONTROLDP0         (SDCARD_AUX_CONTROL_DP0_REG)
#define SDCARD_TXBUFFERREAD           (SDCARD_txBufferRead)
#define SDCARD_TXBUFFERWRITE          (SDCARD_txBufferWrite)
#define SDCARD_RXBUFFERREAD           (SDCARD_rxBufferRead)
#define SDCARD_RXBUFFERWRITE          (SDCARD_rxBufferWrite)

#define SDCARD_COUNTER_PERIOD         (SDCARD_COUNTER_PERIOD_REG)
#define SDCARD_COUNTER_CONTROL        (SDCARD_COUNTER_CONTROL_REG)
#define SDCARD_STATUS                 (SDCARD_TX_STATUS_REG)
#define SDCARD_CONTROL                (SDCARD_CONTROL_REG)
#define SDCARD_STATUS_MASK            (SDCARD_TX_STATUS_MASK_REG)
#define SDCARD_STATUS_ACTL            (SDCARD_TX_STATUS_ACTL_REG)

#define SDCARD_INIT_INTERRUPTS_MASK  (SDCARD_INT_ON_SPI_DONE     | \
                                                SDCARD_INT_ON_TX_EMPTY     | \
                                                SDCARD_INT_ON_TX_NOT_FULL_DEF  | \
                                                SDCARD_INT_ON_RX_FULL      | \
                                                SDCARD_INT_ON_RX_NOT_EMPTY | \
                                                SDCARD_INT_ON_RX_OVER      | \
                                                SDCARD_INT_ON_BYTE_COMP)
                                                
#define SDCARD_DataWidth                  (SDCARD_DATA_WIDTH)
#define SDCARD_InternalClockUsed          (SDCARD_INTERNAL_CLOCK)
#define SDCARD_InternalTxInterruptEnabled (SDCARD_INTERNAL_TX_INT_ENABLED)
#define SDCARD_InternalRxInterruptEnabled (SDCARD_INTERNAL_RX_INT_ENABLED)
#define SDCARD_ModeUseZero                (SDCARD_MODE_USE_ZERO)
#define SDCARD_BidirectionalMode          (SDCARD_BIDIRECTIONAL_MODE)
#define SDCARD_Mode                       (SDCARD_MODE)
#define SDCARD_DATAWIDHT                  (SDCARD_DATA_WIDTH)
#define SDCARD_InternalInterruptEnabled   (0u)

#define SDCARD_TXBUFFERSIZE   (SDCARD_TX_BUFFER_SIZE)
#define SDCARD_RXBUFFERSIZE   (SDCARD_RX_BUFFER_SIZE)

#define SDCARD_TXBUFFER       SDCARD_txBuffer
#define SDCARD_RXBUFFER       SDCARD_rxBuffer

#endif /* (CY_SPIM_SDCARD_H) */


/* [] END OF FILE */
