/*******************************************************************************
* File Name: SDCARD_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SDCARD_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START SDCARD_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: SDCARD_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  SDCARD_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  SDCARD_txBuffer[SDCARD_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SDCARD_TX_ISR)
{
    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef SDCARD_TX_ISR_ENTRY_CALLBACK
        SDCARD_TX_ISR_EntryCallback();
    #endif /* SDCARD_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START SDCARD_TX_ISR_START` */

    /* `#END` */
    
    #if(SDCARD_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(SDCARD_txBufferRead != SDCARD_txBufferWrite)
        {
            tmpStatus = SDCARD_GET_STATUS_TX(SDCARD_swStatusTx);
            SDCARD_swStatusTx = tmpStatus;

            if(0u != (SDCARD_swStatusTx & SDCARD_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == SDCARD_txBufferFull)
                {
                   SDCARD_txBufferRead++;

                    if(SDCARD_txBufferRead >= SDCARD_TX_BUFFER_SIZE)
                    {
                        SDCARD_txBufferRead = 0u;
                    }
                }
                else
                {
                    SDCARD_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(SDCARD_TXDATA_PTR, 
                                             SDCARD_txBuffer[SDCARD_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(SDCARD_txBufferRead == SDCARD_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            SDCARD_TX_STATUS_MASK_REG &= ((uint8) ~SDCARD_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (SDCARD_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SDCARD_TX_ISR_END` */

    /* `#END` */
    
    #ifdef SDCARD_TX_ISR_EXIT_CALLBACK
        SDCARD_TX_ISR_ExitCallback();
    #endif /* SDCARD_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: SDCARD_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  SDCARD_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  SDCARD_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  SDCARD_rxBuffer[SDCARD_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(SDCARD_RX_ISR)
{
    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef SDCARD_RX_ISR_ENTRY_CALLBACK
        SDCARD_RX_ISR_EntryCallback();
    #endif /* SDCARD_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START SDCARD_RX_ISR_START` */

    /* `#END` */
    
    #if(SDCARD_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = SDCARD_GET_STATUS_RX(SDCARD_swStatusRx);
        SDCARD_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (SDCARD_swStatusRx & SDCARD_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(SDCARD_RXDATA_PTR);

            /* Set next pointer. */
            SDCARD_rxBufferWrite++;
            if(SDCARD_rxBufferWrite >= SDCARD_RX_BUFFER_SIZE)
            {
                SDCARD_rxBufferWrite = 0u;
            }

            if(SDCARD_rxBufferWrite == SDCARD_rxBufferRead)
            {
                SDCARD_rxBufferRead++;
                if(SDCARD_rxBufferRead >= SDCARD_RX_BUFFER_SIZE)
                {
                    SDCARD_rxBufferRead = 0u;
                }

                SDCARD_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            SDCARD_rxBuffer[SDCARD_rxBufferWrite] = rxData;

            tmpStatus = SDCARD_GET_STATUS_RX(SDCARD_swStatusRx);
            SDCARD_swStatusRx = tmpStatus;
        }

    #endif /* (SDCARD_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START SDCARD_RX_ISR_END` */

    /* `#END` */
    
    #ifdef SDCARD_RX_ISR_EXIT_CALLBACK
        SDCARD_RX_ISR_ExitCallback();
    #endif /* SDCARD_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
